/*
 *************
 * Copyright *
 *************
 *
 * Copyright (C) 2010 Christoph Zauner
 */

/*
 ***********
 * License *
 ***********
 *
 * This file is part of Rihamark.
 * 
 * Rihamark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Rihamark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Rihamark.  If not, see <http://www.gnu.org/licenses/>.
 */

package rmk.core;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Manages the file list which the algorithms are used on.
 *
 * @author christoph
 */
public class Filer {

private File imgDirOriginals;
private File imgDirAttacked;
private LinkedList<ImageEntity> files = new LinkedList<ImageEntity>();
/**
 * Contains the valid file extensions.
 */
private HashSet<String> validExtensions;

/**
 * Creates a "file list" from a tests "Image Directory" and
 * "Attack Image Directory". The path strings for the
 * "Attack Image Directory" are derived from the path strings of the
 * "Image Directory".
 *
 * The "file list" is a <code>LinkedList</code> containing
 * <code>ImageEntity</code> objects. The access to
 * <code>ImageEntity</code> objects is encapsulated by this class.
 * Other classes can access the <code>ImageEntity</code> objects by
 * their index number.
 *
 * @param test
 */
public Filer(Test test) {
	imgDirOriginals = new File(test.getImageDir());
	imgDirAttacked = new File(test.getImageAttackDir());

	validExtensions = new HashSet<String>();
	validExtensions.add("bmp");
	validExtensions.add("gif");
	validExtensions.add("jpg");
	validExtensions.add("png");
	validExtensions.add("tif");

	try {
		findFiles(imgDirOriginals);
	} catch (IOException ex) {
		Logger.getLogger(Filer.class.getName()).log(Level.SEVERE, null, ex);
	}
}

protected void printFileList() {
	System.out.println("\n--- File List Begin ---\n");
	for(ImageEntity imageEntity: files) {
		System.out.println(imageEntity.pathOriginalImg);
		System.out.println(imageEntity.pathAttackedImg);
	}
	System.out.println("\n--- File List End ---\n");
}

/**
 * Creates a new <code>ImageEntity</code>.
 *
 * @param path
 */
private void addFile(File path) {
	/*
	 * Check if image format and file extentension are supported.
	 * For a list of supported image formats and file
	 * extentensions that are supported by ImageJ see
	 * method saveAs(ImagePlus imp, String format, String path)
	 * in IJ.java.
	 */
	String fileName = path.getName();
	String extension;
	int dotLoc = fileName.lastIndexOf('.');
	if (dotLoc != -1) {
		extension = fileName.substring(dotLoc+1);
		if(!validateExtension(extension)) {
			Debug.debugPrint(Debug.LogLevel.WARN, "Ignoring file >" + path + "< because it has an invalid file extension.");
			return;
		}
	} else {
		Debug.debugPrint(Debug.LogLevel.WARN, "Ignoring file >" + path + "< because it has no file extension.");
		return;
	}

	// derive pathAttackedImg (String pathB) from Strings pathA and baseDirA.
 	String pathA = path.toString();
	String baseDirA = imgDirOriginals.toString();
	int baseDirLen = baseDirA.length();
	String pathB = imgDirAttacked + pathA.substring(baseDirLen);
 
	ImageEntity imageEntity = new ImageEntity(path, new File(pathB));
	files.add(imageEntity);
}

private boolean validateExtension(String extension) {
	boolean validExtension = false;

	if(validExtensions.contains(extension)) {
		validExtension = true;
	}

	return validExtension;
}

/**
 * Calls <code>addFile()</code> on all the files found in <path>path</path>.
 *
 * @param path
 * @throws IOException
 */
private void findFiles(File path) throws IOException {
	//Check to ensure that the path is valid
	if (!path.exists()) {
		throw new IOException();
	//check to ensure we have rights to the source
	} else if (!path.canRead()) {
		throw new IOException();
	}

	// is this a directory?
	if (path.isDirectory()) {
		//get a listing of files
		String fileList[] = path.list();
		// process all the files in the list.
		for (int i = 0; i < fileList.length; i++)
		{
			findFiles(new File(path.toString() + File.separator + fileList[i]));
		}
	//This was not a directory, so lets process the file
	} else {
		addFile(path);
	}
}

protected int getNumOfFiles() {
	return files.size();
}

protected File getFileOriginal(int idx) {
	return files.get(idx).pathOriginalImg;
}

protected File getFileAttacked(int idx) {
	return files.get(idx).pathAttackedImg;
}

void addFileOriginalHash(int idx, Hash originalHash) {
	files.get(idx).HashOriginalImg = originalHash;
}

void addFileAttackedHash(int idx, Hash attackedHash) {
	files.get(idx).HashAttackedImg = attackedHash;
}

protected Hash getFileOriginalHash(int idx) {
	return files.get(idx).HashOriginalImg;
}

protected Hash getFileAttackedHash(int idx) {
	return files.get(idx).HashAttackedImg;
}

} // end of class "Filer"

/**
 * Represents one image entity.
 */
class ImageEntity {

protected File pathOriginalImg;
protected Hash HashOriginalImg;
protected File pathAttackedImg;
protected Hash HashAttackedImg;

public ImageEntity(File pathOriginalImg, File pathAttackedImg) {
	this.pathOriginalImg = pathOriginalImg;
	this.pathAttackedImg = pathAttackedImg;
}

} // end of class of class "ImageEntity"

